@@ -14,14 +14,7 @@
 
 import argparse
 import codecs
-import locale
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
-import sys
-
-import six
+import io
 from unittest import mock
 
 from cliff import app as application
@@ -30,6 +23,7 @@ from cliff import commandmanager
 from cliff.tests import base
 from cliff.tests import utils as test_utils
 from cliff import utils
+import sys
 
 
 def make_app(**kwargs):
@@ -51,6 +45,24 @@ def make_app(**kwargs):
     err_command.return_value = err_command_inst
     cmd_mgr.add_command('error', err_command)
 
+    # Register a command that is interrrupted
+    interrupt_command = mock.Mock(name='interrupt_command', spec=c_cmd.Command)
+    interrupt_command_inst = mock.Mock(spec=c_cmd.Command)
+    interrupt_command_inst.run = mock.Mock(
+        side_effect=KeyboardInterrupt
+    )
+    interrupt_command.return_value = interrupt_command_inst
+    cmd_mgr.add_command('interrupt', interrupt_command)
+
+    # Register a command that is interrrupted by a broken pipe
+    pipeclose_command = mock.Mock(name='pipeclose_command', spec=c_cmd.Command)
+    pipeclose_command_inst = mock.Mock(spec=c_cmd.Command)
+    pipeclose_command_inst.run = mock.Mock(
+        side_effect=BrokenPipeError
+    )
+    pipeclose_command.return_value = pipeclose_command_inst
+    cmd_mgr.add_command('pipe-close', pipeclose_command)
+
     app = application.App('testing interactive mode',
                           '1',
                           cmd_mgr,
@@ -113,6 +125,16 @@ class TestInitAndCleanup(base.TestBase):
         app.run(['mock'])
         app.prepare_to_run_command.assert_called_once_with(command())
 
+    def test_interrupt_command(self):
+        app, command = make_app()
+        result = app.run(['interrupt'])
+        self.assertEqual(result, 130)
+
+    def test_pipeclose_command(self):
+        app, command = make_app()
+        result = app.run(['pipe-close'])
+        self.assertEqual(result, 141)
+
     def test_clean_up_success(self):
         app, command = make_app()
         app.clean_up = mock.MagicMock(name='clean_up')
@@ -148,6 +170,32 @@ class TestInitAndCleanup(base.TestBase):
         self.assertIsInstance(args[2], RuntimeError)
         self.assertEqual(('test exception',), args[2].args)
 
+    def test_clean_up_interrupt(self):
+        app, command = make_app()
+
+        app.clean_up = mock.MagicMock(name='clean_up')
+        ret = app.run(['interrupt'])
+        self.assertNotEqual(ret, 0)
+
+        app.clean_up.assert_called_once_with(mock.ANY, mock.ANY, mock.ANY)
+        call_args = app.clean_up.call_args_list[0]
+        self.assertEqual(mock.call(mock.ANY, 130, mock.ANY), call_args)
+        args, kwargs = call_args
+        self.assertIsInstance(args[2], KeyboardInterrupt)
+
+    def test_clean_up_pipeclose(self):
+        app, command = make_app()
+
+        app.clean_up = mock.MagicMock(name='clean_up')
+        ret = app.run(['pipe-close'])
+        self.assertNotEqual(ret, 0)
+
+        app.clean_up.assert_called_once_with(mock.ANY, mock.ANY, mock.ANY)
+        call_args = app.clean_up.call_args_list[0]
+        self.assertEqual(mock.call(mock.ANY, 141, mock.ANY), call_args)
+        args, kwargs = call_args
+        self.assertIsInstance(args[2], BrokenPipeError)
+
     def test_error_handling_clean_up_raises_exception(self):
         app, command = make_app()
 
@@ -273,7 +321,7 @@ class TestOptionParser(base.TestBase):
                 return parser
 
             def take_action(self, parsed_args):
-                assert(parsed_args.end == '123')
+                assert parsed_args.end == '123'
 
         class MyCommandManager(commandmanager.CommandManager):
             def load_commands(self, namespace):
@@ -322,6 +370,31 @@ class TestHelpHandling(base.TestBase):
     def test_deferred_help(self):
         self._test_help(True)
 
+    def _test_interrupted_help(self, deferred_help):
+        app, _ = make_app(deferred_help=deferred_help)
+        with mock.patch('cliff.help.HelpAction.__call__',
+                        side_effect=KeyboardInterrupt):
+            result = app.run(['--help'])
+            self.assertEqual(result, 130)
+
+    def test_interrupted_help(self):
+        self._test_interrupted_help(False)
+
+    def test_interrupted_deferred_help(self):
+        self._test_interrupted_help(True)
+
+    def _test_pipeclose_help(self, deferred_help):
+        app, _ = make_app(deferred_help=deferred_help)
+        with mock.patch('cliff.help.HelpAction.__call__',
+                        side_effect=BrokenPipeError):
+            app.run(['--help'])
+
+    def test_pipeclose_help(self):
+        self._test_pipeclose_help(False)
+
+    def test_pipeclose_deferred_help(self):
+        self._test_pipeclose_help(True)
+
     def test_subcommand_help(self):
         app, _ = make_app(deferred_help=False)
 
@@ -355,7 +428,7 @@ class TestCommandLookup(base.TestBase):
             self.assertIn("['hell']", str(err))
 
     def test_list_matching_commands(self):
-        stdout = StringIO()
+        stdout = io.StringIO()
         app = application.App('testing', '1',
                               test_utils.TestCommandManager(
                                   test_utils.TEST_NAMESPACE),
@@ -433,133 +506,41 @@ class TestIO(base.TestBase):
         cmd_mgr = commandmanager.CommandManager('cliff.tests')
         io = mock.Mock()
 
-        if six.PY2:
-            stdin_save = sys.stdin
-            stdout_save = sys.stdout
-            stderr_save = sys.stderr
-            encoding = locale.getpreferredencoding() or 'utf-8'
+        app = application.App('no io streams', 1, cmd_mgr)
+        self.assertIs(sys.stdin, app.stdin)
+        self.assertIs(sys.stdout, app.stdout)
+        self.assertIs(sys.stderr, app.stderr)
 
-            app = application.App('no io streams', 1, cmd_mgr)
-            self.assertIsInstance(app.stdin, codecs.StreamReader)
-            self.assertIsInstance(app.stdout, codecs.StreamWriter)
-            self.assertIsInstance(app.stderr, codecs.StreamWriter)
+        app = application.App('with stdin io stream', 1, cmd_mgr, stdin=io)
+        self.assertIs(io, app.stdin)
+        self.assertIs(sys.stdout, app.stdout)
+        self.assertIs(sys.stderr, app.stderr)
 
-            app = application.App('with stdin io stream', 1, cmd_mgr, stdin=io)
-            self.assertIs(io, app.stdin)
-            self.assertIsInstance(app.stdout, codecs.StreamWriter)
-            self.assertIsInstance(app.stderr, codecs.StreamWriter)
+        app = application.App('with stdout io stream', 1, cmd_mgr,
+                              stdout=io)
+        self.assertIs(sys.stdin, app.stdin)
+        self.assertIs(io, app.stdout)
+        self.assertIs(sys.stderr, app.stderr)
 
-            app = application.App('with stdout io stream', 1, cmd_mgr,
-                                  stdout=io)
-            self.assertIsInstance(app.stdin, codecs.StreamReader)
-            self.assertIs(io, app.stdout)
-            self.assertIsInstance(app.stderr, codecs.StreamWriter)
-
-            app = application.App('with stderr io stream', 1, cmd_mgr,
-                                  stderr=io)
-            self.assertIsInstance(app.stdin, codecs.StreamReader)
-            self.assertIsInstance(app.stdout, codecs.StreamWriter)
-            self.assertIs(io, app.stderr)
-
-            try:
-                sys.stdin = codecs.getreader(encoding)(sys.stdin)
-                app = application.App(
-                    'with wrapped sys.stdin io stream', 1, cmd_mgr)
-                self.assertIs(sys.stdin, app.stdin)
-                self.assertIsInstance(app.stdout, codecs.StreamWriter)
-                self.assertIsInstance(app.stderr, codecs.StreamWriter)
-            finally:
-                sys.stdin = stdin_save
-
-            try:
-                sys.stdout = codecs.getwriter(encoding)(sys.stdout)
-                app = application.App('with wrapped stdout io stream', 1,
-                                      cmd_mgr)
-                self.assertIsInstance(app.stdin, codecs.StreamReader)
-                self.assertIs(sys.stdout, app.stdout)
-                self.assertIsInstance(app.stderr, codecs.StreamWriter)
-            finally:
-                sys.stdout = stdout_save
-
-            try:
-                sys.stderr = codecs.getwriter(encoding)(sys.stderr)
-                app = application.App('with wrapped stderr io stream', 1,
-                                      cmd_mgr)
-                self.assertIsInstance(app.stdin, codecs.StreamReader)
-                self.assertIsInstance(app.stdout, codecs.StreamWriter)
-                self.assertIs(sys.stderr, app.stderr)
-            finally:
-                sys.stderr = stderr_save
-
-        else:
-            app = application.App('no io streams', 1, cmd_mgr)
-            self.assertIs(sys.stdin, app.stdin)
-            self.assertIs(sys.stdout, app.stdout)
-            self.assertIs(sys.stderr, app.stderr)
-
-            app = application.App('with stdin io stream', 1, cmd_mgr, stdin=io)
-            self.assertIs(io, app.stdin)
-            self.assertIs(sys.stdout, app.stdout)
-            self.assertIs(sys.stderr, app.stderr)
-
-            app = application.App('with stdout io stream', 1, cmd_mgr,
-                                  stdout=io)
-            self.assertIs(sys.stdin, app.stdin)
-            self.assertIs(io, app.stdout)
-            self.assertIs(sys.stderr, app.stderr)
-
-            app = application.App('with stderr io stream', 1, cmd_mgr,
-                                  stderr=io)
-            self.assertIs(sys.stdin, app.stdin)
-            self.assertIs(sys.stdout, app.stdout)
-            self.assertIs(io, app.stderr)
+        app = application.App('with stderr io stream', 1, cmd_mgr,
+                              stderr=io)
+        self.assertIs(sys.stdin, app.stdin)
+        self.assertIs(sys.stdout, app.stdout)
+        self.assertIs(io, app.stderr)
 
     def test_writer_encoding(self):
         # The word "test" with the e replaced by
         # Unicode latin small letter e with acute,
         # U+00E9, utf-8 encoded as 0xC3 0xA9
-        text = u't\u00E9st'
+        text = 't\u00E9st'
         text_utf8 = text.encode('utf-8')
 
-        if six.PY2:
-            # In PY2 StreamWriter can't accept non-ASCII encoded characters
-            # because it must first promote the encoded byte stream to
-            # unicode in order to encode it in the desired encoding.
-            # Because the encoding of the byte stream is not known at this
-            # point the default-encoding of ASCII is utilized, but you can't
-            # decode a non-ASCII charcater to ASCII.
-            io = six.StringIO()
-            writer = codecs.getwriter('utf-8')(io)
-            self.assertRaises(UnicodeDecodeError,
-                              writer.write,
-                              text_utf8)
+        # In PY3 you can't write encoded bytes to a text writer
+        # instead text functions require text.
+        out = io.StringIO()
+        writer = codecs.getwriter('utf-8')(out)
+        self.assertRaises(TypeError, writer.write, text)
 
-            # In PY2 with our override of codecs.getwriter we do not
-            # attempt to encode bytes in a str object (only unicode
-            # objects) therefore the final output string should be the
-            # utf-8 encoded byte sequence
-            io = six.StringIO()
-            writer = utils.getwriter('utf-8')(io)
-            writer.write(text)
-            output = io.getvalue()
-            self.assertEqual(text_utf8, output)
-
-            io = six.StringIO()
-            writer = utils.getwriter('utf-8')(io)
-            writer.write(text_utf8)
-            output = io.getvalue()
-            self.assertEqual(text_utf8, output)
-        else:
-            # In PY3 you can't write encoded bytes to a text writer
-            # instead text functions require text.
-            io = six.StringIO()
-            writer = utils.getwriter('utf-8')(io)
-            self.assertRaises(TypeError,
-                              writer.write,
-                              text)
-
-            io = six.StringIO()
-            writer = utils.getwriter('utf-8')(io)
-            self.assertRaises(TypeError,
-                              writer.write,
-                              text_utf8)
+        out = io.StringIO()
+        writer = codecs.getwriter('utf-8')(out)
+        self.assertRaises(TypeError, writer.write, text_utf8)
